[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# IRN Contacts client library for Python

[![API Version](https://img.shields.io/badge/api-v1-blue)](https://developer.factset.com/api-catalog/irn-contacts-api#apiDocumentation)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.IRNContacts)](https://pypi.org/project/fds.sdk.IRNContacts/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Allows users to extract, create, update and configure IRN data.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1
- SDK version: 1.1.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.IRNContacts==1.1.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.IRNContacts==1.1.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.IRNContacts
from fds.sdk.IRNContacts.api import contacts_api
from fds.sdk.IRNContacts.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.IRNContacts.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.IRNContacts.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.IRNContacts.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = contacts_api.ContactsApi(api_client)
    contact_save_dto = ContactSaveDto(
        primary_email_address="primary_email_address_example",
        full_name="full_name_example",
        notes="notes_example",
        linked_in_profile="linked_in_profile_example",
        person_symbol="person_symbol_example",
        role_id="role_id_example",
        type_id="type_id_example",
        address=ContactAddressDto(
            google_maps_place_id="google_maps_place_id_example",
            formatted_address="formatted_address_example",
            city="city_example",
            state_province="state_province_example",
            postal_code="postal_code_example",
            country="country_example",
        ),
        phone_numbers=[
            ContactPhoneSaveDto(
                id="id_example",
                number="number_example",
                type_id="type_id_example",
                is_primary=True,
            ),
        ],
        left_relationships=[
            ContactRelationshipSaveDto(
                relationship_id="relationship_id_example",
                contact_id="contact_id_example",
                start=dateutil_parser('1970-01-01T00:00:00.00Z'),
                end=dateutil_parser('1970-01-01T00:00:00.00Z'),
                comment="comment_example",
                is_current=True,
                symbol="symbol_example",
                title="title_example",
            ),
        ],
        right_relationships=[
            ContactRelationshipSaveDto(
                relationship_id="relationship_id_example",
                contact_id="contact_id_example",
                start=dateutil_parser('1970-01-01T00:00:00.00Z'),
                end=dateutil_parser('1970-01-01T00:00:00.00Z'),
                comment="comment_example",
                is_current=True,
                symbol="symbol_example",
                title="title_example",
            ),
        ],
        alternative_email_addresses=[
            AlternativeEmailAddressDto(
                id="id_example",
                email_address="email_address_example",
            ),
        ],
        custom_fields=[
            ContactCustomFieldValueSaveDto(
                code="code_example",
                value="value_example",
                option_values=[
                    "option_values_example",
                ],
                evaluate_formula=False,
            ),
        ],
    ) # ContactSaveDto | contactSaveDto object to save (optional)

    try:
        # Create a contact
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.create_contact(contact_save_dto=contact_save_dto)

        pprint(api_response)
    except fds.sdk.IRNContacts.ApiException as e:
        print("Exception when calling ContactsApi->create_contact: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Create a contact
    #     api_response, http_status_code, response_headers = api_instance.create_contact_with_http_info(contact_save_dto=contact_save_dto)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.IRNContacts.ApiException as e:
    #     print("Exception when calling ContactsApi->create_contact: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Create a contact
    #     async_result = api_instance.create_contact_async(contact_save_dto=contact_save_dto)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.IRNContacts.ApiException as e:
    #     print("Exception when calling ContactsApi->create_contact: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Create a contact
    #     async_result = api_instance.create_contact_with_http_info_async(contact_save_dto=contact_save_dto)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.IRNContacts.ApiException as e:
    #     print("Exception when calling ContactsApi->create_contact: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.IRNContacts

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.IRNContacts.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.IRNContacts

configuration = fds.sdk.IRNContacts.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.IRNContacts

configuration = fds.sdk.IRNContacts.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.IRNContacts

configuration = fds.sdk.IRNContacts.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/research/irn/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*ContactsApi* | [**create_contact**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#create_contact) | **POST** /contacts | Create a contact
*ContactsApi* | [**delete_contact**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#delete_contact) | **DELETE** /contacts/{contactId} | Delete a contact
*ContactsApi* | [**get_contact**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contact) | **GET** /contacts/{contactId} | Get all custom field and standard field details on a specific contact
*ContactsApi* | [**get_contact_events**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contact_events) | **GET** /contacts/{contactId}/events | Get a contact’s audit history
*ContactsApi* | [**get_contact_notes**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contact_notes) | **GET** /contacts/{contactId}/about | Get the About field content for a specific contact
*ContactsApi* | [**get_contact_records**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contact_records) | **GET** /contacts/{contactId}/records | Get all notes and meetings where a specific contact was tagged
*ContactsApi* | [**get_contact_relationships**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contact_relationships) | **GET** /contacts/{contactId}/relationships | Returns a list of a contact’s relationships
*ContactsApi* | [**get_contacts**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#get_contacts) | **GET** /contacts | Get list of all contacts in your group along with some of their standard field data
*ContactsApi* | [**patch_contact**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsApi.md#patch_contact) | **PATCH** /contacts/{contactId} | Edit a contact’s standard field and custom field data
*ContactsRelationshipsApi* | [**create_contact_relationship**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsRelationshipsApi.md#create_contact_relationship) | **POST** /contact-relationships | Create a contact relationship
*ContactsRelationshipsApi* | [**delete_contact_relationship**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsRelationshipsApi.md#delete_contact_relationship) | **DELETE** /contact-relationships/{contactRelationshipId} | Delete a contact relationship
*ContactsRelationshipsApi* | [**get_all_relationships_tagging_a_contact**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsRelationshipsApi.md#get_all_relationships_tagging_a_contact) | **GET** /contact-relationships | Get all the relationships where the given symbol or contact identifier has been tagged in a relationship
*ContactsRelationshipsApi* | [**get_contact_relationship**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsRelationshipsApi.md#get_contact_relationship) | **GET** /contact-relationships/{contactRelationshipId} | Get details on a specific contact relationship
*ContactsRelationshipsApi* | [**patch_contact_relationship**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactsRelationshipsApi.md#patch_contact_relationship) | **PATCH** /contact-relationships/{contactRelationshipId} | Update a contact relationship


## Documentation For Models

 - [AlternativeEmailAddressDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/AlternativeEmailAddressDto.md)
 - [ContactAddressDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactAddressDto.md)
 - [ContactCustomFieldOptionValueDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactCustomFieldOptionValueDto.md)
 - [ContactCustomFieldValueDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactCustomFieldValueDto.md)
 - [ContactCustomFieldValueSaveDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactCustomFieldValueSaveDto.md)
 - [ContactDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactDto.md)
 - [ContactEventDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactEventDto.md)
 - [ContactEventUpdateDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactEventUpdateDto.md)
 - [ContactIdNameDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactIdNameDto.md)
 - [ContactPhoneDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactPhoneDto.md)
 - [ContactPhoneSaveDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactPhoneSaveDto.md)
 - [ContactRelationshipDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactRelationshipDto.md)
 - [ContactRelationshipSaveDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactRelationshipSaveDto.md)
 - [ContactRelationshipTwoSidedDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactRelationshipTwoSidedDto.md)
 - [ContactRelationshipTwoSidedSaveDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactRelationshipTwoSidedSaveDto.md)
 - [ContactRoleDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactRoleDto.md)
 - [ContactSaveDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactSaveDto.md)
 - [ContactSummaryDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactSummaryDto.md)
 - [ContactTypeDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ContactTypeDto.md)
 - [EmployerDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/EmployerDto.md)
 - [NewItemDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/NewItemDto.md)
 - [Operation](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/Operation.md)
 - [OperationType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/OperationType.md)
 - [PhoneNumberTypeDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/PhoneNumberTypeDto.md)
 - [ProblemDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/ProblemDetails.md)
 - [RecordPreviewDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/RecordPreviewDto.md)
 - [RelationshipCategoryIdNameDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/RelationshipCategoryIdNameDto.md)
 - [RelationshipIdNameDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/RelationshipIdNameDto.md)
 - [RelationshipSymbolSummaryDto](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/RelationshipSymbolSummaryDto.md)
 - [SymbolType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/IRNContacts/v1/docs/SymbolType.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.IRNContacts.apis and fds.sdk.IRNContacts.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.IRNContacts.api.default_api import DefaultApi`
- `from fds.sdk.IRNContacts.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.IRNContacts
from fds.sdk.IRNContacts.apis import *
from fds.sdk.IRNContacts.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

